from typing import Tuple
import matplotlib.pyplot as plt

# Parameters to match Google Slides template
GOLDEN_SLIDE = dict(width=490)
FULL_SLIDE = dict(width=625, ratio=0.48)
TWO_THIRDS_SLIDE = dict(width=430)
BOTTOM_SLIDE = dict(width=625, ratio=(5**0.5 - 1) / 4)


def get_dim(
    width: float,
    fraction_of_line_width: float = 1,
    ratio: float = (5**0.5 - 1) / 2,
) -> Tuple[float, float]:
    """
    Return figure height, width in inches to avoid scaling in latex.

    Default aspect ratio is the golden ratio, and a figure size occupying
    full page width. To get the correct text width for your latex document,
    check:
    https://tex.stackexchange.com/questions/39383/determine-text-width

    # TODO:  Add Visium LaTeX report default width and default width for

    Args:
      width (float): Textwidth of the report (in pt) to make fontsizes match.
      fraction_of_line_width (float, optional): Fraction of the document width
        which you wish the figure to occupy. Use e.g. 0.5 for a figure which should
        only fill one column in a double column layout. Defaults to 1.
      ratio (float, optional): Fraction of figure width to figure height.
        Defaults to (5 ** 0.5 - 1)/2.
    Returns:
      fig_dim (tuple):
          Dimensions of figure in inches meant to be passed on directly to
          matplotlib via the `figsize` argument.
    """

    # Width of figure
    fig_width_pt = width * fraction_of_line_width

    # Convert from pt to inches
    inches_per_pt = 1 / 72.27

    # Figure width in inches
    fig_width_in = fig_width_pt * inches_per_pt
    # Figure height in inches
    fig_height_in = fig_width_in * ratio

    fig_dim = (fig_width_in, fig_height_in)

    return fig_dim


def set_dim(
    fig: plt.figure,
    width: float = GOLDEN_SLIDE["width"],
    fraction_of_line_width: float = 1,
    ratio: float = (5**0.5 - 1) / 2,
) -> None:
    """
    Set aesthetic figure dimensions to avoid scaling in latex.

    Args:
      fig (plt.figure): Figure object to resize.
      width (float): Textwidth of the report to make fontsizes match.
      fraction_of_line_width (float, optional): Fraction of the document width
        which you wish the figure to occupy.  Defaults to 1.
      ratio (float, optional): Fraction of figure width that the figure height
        should be. Defaults to (5 ** 0.5 - 1)/2.
    Returns:
      None: alters current figure to have the desired dimensions
    """
    fig.set_size_inches(
        get_dim(width=width, fraction_of_line_width=fraction_of_line_width, ratio=ratio)
    )



import os
import warnings

import plotly
import plotly.express as px
import plotly.graph_objects as go

from .dims import GOLDEN_SLIDE, set_dim

filepath = os.path.dirname(os.path.abspath(__file__))


def _format(
    fig: plotly.graph_objs._figure.Figure,
    width: float = GOLDEN_SLIDE["width"],
    fraction_of_line_width: float = 1,
    ratio: float = (5**0.5 - 1) / 2,
) -> None:
    """
    Fetches information from current pyplot to verify and impose format.

    Args:
        plt (matplotlib.pyplot): Pyplot object
        width (float): Textwidth of the report to make fontsizes match.
        fraction_of_line_width (float, optional): Fraction of the document width
            which you wish the figure to occupy.  Defaults to 1.
        ratio (float, optional): Fraction of figure width that the figure height
            should be. Defaults to (5 ** 0.5 - 1)/2.
    Returns:
        None: alters plt to ensure good formatting.
    """

    if fig.layout.title.text is None:
        warnings.warn("Title is not specified!")
    if fig.layout.xaxis.title.text is None:
        warnings.warn("X-axis label not specified!")
    if fig.layout.yaxis.title.text is None:
        warnings.warn("Y-axis label not specified!")

    if fig.layout.title.text is not None:
        new_title = " ".join(fig.layout.title.text.split("_")).capitalize()
        fig.update_layout(title=new_title)
    if fig.layout.xaxis.title.text is not None:
        new_xlabel = " ".join(fig.layout.xaxis.title.text.split("_")).capitalize()
        fig.update_layout(xaxis_title=new_xlabel)
    if fig.layout.yaxis.title.text is not None:
        new_ylabel = " ".join(fig.layout.yaxis.title.text.split("_")).capitalize()
        fig.update_layout(yaxis_title=new_ylabel)

    fig.update_layout(
        width=width * fraction_of_line_width * 1.38,
        height=width * ratio * 1.38,
        margin=dict(l=10, r=10, t=40, b=10),
        font=dict(family="Arial", size=15),
    )


plotly.format = _format



import seaborn as sns
import matplotlib

from typing import Callable
from functools import wraps


def _instantiate(
    f: Callable[..., matplotlib.axes.Axes]
) -> matplotlib.axes.Axes:
    """
    Instantiate function redefinition by wrapper.
    
    Args:
        f (Callable[..., matplotlib.axes.Axes]): Axes object to instantiate
    Returns:
        matplotlib.axes.Axes: instantiated Axes object
    """
    @wraps(f)
    def wrapper(*args, **kwargs):
        
        return f()(*args, **kwargs)
    
    return wrapper


def _confusion_matrix(
    f: Callable[..., matplotlib.axes.Axes]
) -> matplotlib.axes.Axes:
    """
    Instantiate Axes with defaults attributed to Confusion Matrix.
    
    Args:
        f (Callable[..., matplotlib.axes.Axes]): Axes object to instantiate
    Returns:
        matplotlib.axes.Axes: instantiated Axes object
    """
    @wraps(f)
    def wrapper(*args, **kwargs):
        
        ax = f()(*args, **kwargs)
        ax.set_xlabel('Prediction')
        ax.set_ylabel('Ground truth')
        ax.set_xticklabels(ax.get_xticklabels(), rotation=45)
        ax.set_yticklabels(ax.get_yticklabels(), rotation=45)
        
        return ax
    
    return wrapper


def _color_map(
    f: Callable
) -> Callable:
    """
    Wrapper that modifies default parameters `annot`, `cbar`, and `cmap`
    of Callable, if and only if they are not yet set.

    Args:
        f (Callable): Function with parameters `annot`, `cbar`, and `cmap`.

    Returns:
        Callable: Function with modified default parameters.
    """
    @wraps(f)
    def wrapper(*args, **kwargs):
        
        overridden_args = dict(annot=True, cbar=False, cmap='Blues')
        overridden_args.update(kwargs)
        
        kwargs = overridden_args
        
        return f(*args, **kwargs)
    
    return wrapper


# Store seaborn function in temp
_temp_heatmap = sns.heatmap


# Redefine `seaborn.heatmap`
@_color_map
@_instantiate
def _visium_heatmap(*args, **kwargs):
    return _temp_heatmap


# Define `seaborn.confusion`
@_color_map
@_confusion_matrix
def _visium_confusion(*args, **kwargs):
    return _temp_heatmap


# Allocate functions to `seaborn` module
sns.heatmap = _visium_heatmap
sns.confusion = _visium_confusion



import os
import warnings

import matplotlib as mpl
import matplotlib.pyplot as plt

from .dims import GOLDEN_SLIDE, set_dim

filepath = os.path.dirname(os.path.abspath(__file__))

# Use Visium styling
plt.style.use(filepath + "/visium.mplstyle")


VISIUM_LIGHT = "#5eb2fc"
VISIUM_CLASSIC = "#0858cf"
VISIUM_DARK = "#162b5d"


def _format(
    width: float = GOLDEN_SLIDE["width"],
    fraction_of_line_width: float = 1,
    ratio: float = (5**0.5 - 1) / 2,
) -> None:
    """
    Fetches information from current pyplot to verify and impose format.

    Args:
        plt (matplotlib.pyplot): Pyplot object
        width (float): Textwidth of the report to make fontsizes match.
        fraction_of_line_width (float, optional): Fraction of the document width
            which you wish the figure to occupy.  Defaults to 1.
        ratio (float, optional): Fraction of figure width that the figure height
            should be. Defaults to (5 ** 0.5 - 1)/2.
    Returns:
        None: alters plt to ensure good formatting.
    """

    fig = plt.gcf()
    axes = fig.axes

    for ax in axes:
        if ax.get_title() == "":
            warnings.warn("Title is not specified!")
        if ax.get_xlabel() == "":
            warnings.warn("X-axis label not specified!")
        if ax.get_ylabel() == "":
            warnings.warn("Y-axis label not specified!")

    # Format snake_case to Capitalized case
    new_xlabel = " ".join(ax.get_xlabel().split("_")).capitalize()
    ax.set_xlabel(new_xlabel)

    new_ylabel = " ".join(ax.get_ylabel().split("_")).capitalize()
    ax.set_ylabel(new_ylabel)

    set_dim(
        fig, width=width, fraction_of_line_width=fraction_of_line_width, ratio=ratio
    )

    plt.tight_layout()


# Extend `matplotlib.pyplot` with `_format` method
plt.format = _format



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="visplotlib_test",
    version="1.3.1",
    author="Visium SA",
    author_email="yuxiao.li@visium.ch",
    description="Extensions to Matplotlib and Seaborn plotting for Visium SA.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yetiiil/visplotlib_test",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=["matplotlib", "seaborn", "plotly"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
